{% extends "sitegate/signin/_form_base.html" %}
{% load i18n %}
{% block fields %}
    {% if signin_form.non_field_errors %}
    <div class="alert-box alert">
        <ul class="no-bullet">
            {% for error in signin_form.non_field_errors %}
            <li>{{ error }}</li>
            {% endfor %}
        </ul>
    </div>
    {% endif %}
    {% for field in signin_form %}
        {{ field }}
        {% if field.errors %}<small class="error">{{ field.errors.0 }}</small>{% endif %}
        {% if field.help_text %}<div class="alert-box secondary">{{ field.help_text }}</div>{% endif %}
    {% endfor %}
    <input type="submit" class="button" value="{% trans "Log in" %}" />
{% endblock %}
